<?php

/**
 *
 * @link              https://www.dicha.gr/
 * @since             1.0.0
 * @package           Paybybank
 *
 * @wordpress-plugin
 * Plugin Name:       PayByBank
 * Plugin URI:        https://www.paybybank.gr/el/companies/e-commerce/verified_plugins
 * Description:       Adds PayByBank payment as a payment gateway for WooCommerce.
 * Version:           1.0.2
 * Requires at least: 4.4
 * Tested up to: 5.4
 * WC requires at least: 3.3.0
 * WC tested up to: 4.1.0
 * Author:            Digital Challenge
 * Author URI:        https://www.dicha.gr
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       dc-paybybank
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Check if WooCommerce is active
 **/
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

	/**
	 * Currently plugin version.
	 * Start at version 1.0.0 and use SemVer - https://semver.org
	 * Rename this for your plugin and update it as you release new versions.
	 */
	define( 'DC_PAYBYBANK_VERSION', '1.0.2' );
	define( 'DC_PAYBYBANK_BASE_TEST_PATH', 'https://testapi.e-paylink.com/gateway/rest/api/v1' );
	define( 'DC_PAYBYBANK_BASE_PATH', 'https://www.wu-online.gr/gateway/rest/api/v1' );
	define( 'DC_PAYBYBANK_SUBMIT_ORDER_PATH', '/order/merchant/' );

	/**
	 * The code that runs during plugin activation.
	 * This action is documented in includes/class-dc-paybybank-activator.php
	 */
	function activate_dc_paybybank() {
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-dc-paybybank-activator.php';
		Dc_Paybybank_Activator::activate();
	}

	/**
	 * The code that runs during plugin deactivation.
	 * This action is documented in includes/class-dc-paybybank-deactivator.php
	 */
	function deactivate_dc_paybybank() {
		require_once plugin_dir_path( __FILE__ ) . 'includes/class-dc-paybybank-deactivator.php';
		Dc_Paybybank_Deactivator::deactivate();
	}

	register_activation_hook( __FILE__, 'activate_dc_paybybank' );
	register_deactivation_hook( __FILE__, 'deactivate_dc_paybybank' );

	/**
	 * The core plugin class that is used to define internationalization,
	 * admin-specific hooks, and public-facing site hooks.
	 */
	require plugin_dir_path( __FILE__ ) . 'includes/class-dc-paybybank.php';

	/**
	 * Begins execution of the plugin.
	 *
	 * Since everything within the plugin is registered via hooks,
	 * then kicking off the plugin from this point in the file does
	 * not affect the page life cycle.
	 *
	 * @since    1.0.0
	 */
	function run_dc_paybybank() {

		$plugin = new Dc_Paybybank();
		$plugin->run();

	}

	run_dc_paybybank();

	/**
	 * PayByBank Payment Gateway.
	 *
	 * Creates the PayByBank Payment Gateway.
	 */
	add_action( 'plugins_loaded', 'init_dc_paybybank_gateway_class' );
	function init_dc_paybybank_gateway_class() {

		class WC_Gateway_DC_PayByBank extends WC_Payment_Gateway {

			public $domain;
			public $api_key;
			public $mode;
			public $instructions;
			public $payment_code_life;
			public $order_status;
			public $extra_fee;

			/**
			 * Constructor for the gateway.
			 */
			public function __construct() {

				$this->domain = 'dc-paybybank';

				$this->id                 = 'dc-paybybank';
				$this->icon               = apply_filters( 'woocommerce_paybybank_gateway_icon', '' );
				$this->has_fields         = false;
				$this->method_title       = __( 'PayByBank', $this->domain );
				$this->method_description = __( 'Allows payments with PayByBank.', $this->domain );


				// Load the settings.
				$this->init_form_fields();
				$this->init_settings();

				// Define user set variables
				$this->title             = $this->get_option( 'title' );
				$this->description       = $this->get_option( 'description' );
				$this->instructions      = $this->get_option( 'instructions' );
				$this->api_key           = $this->get_option( 'api_key' );
				$this->mode              = $this->get_option( 'mode' );
				$this->payment_code_life = $this->get_option( 'payment_code_life' );
				$this->order_status      = $this->get_option( 'order_status', 'processing' );
				$this->extra_fee         = $this->get_option( 'extra_fee' );

				// Actions
				add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
					$this,
					'process_admin_options'
				) );

			}

			/**
			 * Initialise Gateway Settings Form Fields.
			 */
			public function init_form_fields() {


				$this->form_fields = array(
					'enabled'           => array(
						'title'   => __( 'Enable/Disable', $this->domain ),
						'type'    => 'checkbox',
						'label'   => __( 'Enable PayByBank Payment', $this->domain ),
						'default' => 'no'
					),
					'title'             => array(
						'title'       => __( 'Title', $this->domain ),
						'type'        => 'text',
						'description' => __( 'This controls the title which the user sees during checkout.', $this->domain ),
						'default'     => __( 'PayByBank Payment', $this->domain ),
					),
					'order_status'      => array(
						'title'       => __( 'Order Status', $this->domain ),
						'type'        => 'select',
						'class'       => 'wc-enhanced-select',
						'description' => __( 'Choose whether status you wish after checkout.', $this->domain ),
						'default'     => 'wc-processing',
						'options'     => wc_get_order_statuses()
					),
					'api_key'           => array(
						'title'       => __( 'API Key', $this->domain ),
						'type'        => 'text',
						'description' => __( 'The API key provided by PayByBank', $this->domain ),
						'default'     => '',
						'desc_tip'    => true,
					),
					'mode'              => array(
						'title'       => __( 'Mode', $this->domain ),
						'type'        => 'checkbox',
						'label'       => __( 'Enable test mode', $this->domain ),
						'description' => __( 'This controls the payment mode as TEST or LIVE.', $this->domain ),
						'default'     => 'yes'
					),
					'payment_code_life' => array(
						'title'       => __( 'Payment Code Life', $this->domain ),
						'type'        => 'text',
						'description' => __( 'The life of Payment Code in Hours. Default is 720 hours (30 days)', $this->domain ),
					),
					'extra_fee'         => array(
						'title'       => __( 'Extra Fee', $this->domain ),
						'type'        => 'text',
						'description' => __( 'Set the extra fee (if applicable) for this payment method. Leave empty if you don\'t want to add extra fee for this payment method', $this->domain ),
					),
					'description'       => array(
						'title'       => __( 'Description', $this->domain ),
						'type'        => 'textarea',
						'description' => __( 'Payment method description that the customer will see on your checkout.', $this->domain ),
						'default'     => __( 'Once you have completed your order, you will receive a PayByBank online payment code. You will receive the password and email you have provided. Then log in with your online banking or telephone banking at any Greek bank you choose PayByBank for payments and enter the code to complete the payment.', $this->domain ),
					),
					'instructions'      => array(
						'title'       => __( 'Instructions', $this->domain ),
						'type'        => 'textarea',
						'description' => __( 'Instructions that will be added to the thank you page and emails.', $this->domain ),
					)
				);
			}

			/**
			 * Process the payment and return the result.
			 *
			 * @param int $order_id
			 *
			 * @return array
			 */
			public function process_payment( $order_id ) {

				$order = wc_get_order( $order_id );
				// Custom function to make the call to API to create the reference code for payment
				$this->create_reference_code( $order );

				$status = 'wc-' === substr( $this->order_status, 0, 3 ) ? substr( $this->order_status, 3 ) : $this->order_status;

				// Set order status
				$order->update_status( $status, __( 'Checkout with PayByBank payment.', $this->domain ) );

				// Reduce stock levels
				wc_reduce_stock_levels( $order_id );

				// Remove cart
				WC()->cart->empty_cart();

				// Return thankyou redirect
				return array(
					'result'   => 'success',
					'redirect' => $this->get_return_url( $order )
				);
			}

			/**
			 * Create reference code for PayByBank from POST request
			 *
			 * @param WC_Order $order
			 */
			public function create_reference_code( WC_Order $order ) {
				if ( $this->mode == 'yes' ) {
					$base_path = DC_PAYBYBANK_BASE_TEST_PATH;
				} else {
					$base_path = DC_PAYBYBANK_BASE_PATH;
				}

				$result = wp_remote_post( $base_path . DC_PAYBYBANK_SUBMIT_ORDER_PATH . $this->api_key .
				                          '?merchant_order_id=' . $order->get_id() . '&merchant_customer_id=' .
				                          $order->get_customer_id() . '&amount=' . $order->get_total() . '&payment_code_life=' . $this->payment_code_life,
					array( 'sslverify' => false ) );
				if ( is_array( $result ) ) {
					$final_result = json_decode( $result['body'], true );
					update_post_meta( $order->get_id(), 'dc_reference_code', $final_result['omtTransactionBank']['bankPaymentCode'] );
				} else {
					error_log( __( 'PayByBank error: Problem during POST request - ', $this->domain ) . $result->get_error_message() );
				}
			}
		}
	}
}